using System;
using System.Collections;
using System.Text;

namespace org.unicode
{	
	/// <summary> Accesses the Normalization Data used for Forms C and D.<br>
	/// Copyright � 1998-1999 Unicode, Inc. All Rights Reserved.<br>
	/// The Unicode Consortium makes no expressed or implied warranty of any
	/// kind, and assumes no liability for errors or omissions.
	/// No liability is assumed for incidental and consequential damages
	/// in connection with or arising out of the use of the information here.
	/// </summary>
	/// <author> Mark Davis
	/// 
	/// </author>
	public class NormalizerData
	{
		private void  InitBlock()
		{
			//UPGRADE_NOTE: Class BitArray does not allow calls to methods with index greater than Length property. 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="jlca1084"'
			isCompatibility = new BitArray(8000); // large enough to hold Viet characters' max value of codepoints
			//UPGRADE_NOTE: Class BitArray does not allow calls to methods with index greater than Length property. 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="jlca1084"'
			isExcluded = new BitArray(0);
		}
		internal const string copyright = "Copyright � 1998-1999 Unicode, Inc.";
		
		/// <summary> Constant for use in getPairwiseComposition
		/// </summary>
		public const int NOT_COMPOSITE = '\uFFFF';
		
		/// <summary> Gets the combining class of a character from the
		/// Unicode Character Database.
		/// </summary>
		/// <param name="ch">     the source character
		/// </param>
		/// <returns>         value from 0 to 255
		/// 
		/// </returns>
		public virtual int getCanonicalClass(char ch)
		{
			return canonicalClass.Get(ch);
		}
		
		/// <summary> Returns the composite of the two characters. If the two
		/// characters don't combine, returns NOT_COMPOSITE.
		/// </summary>
		/// <param name="first">  first character (e.g. 'c')
		/// </param>
		/// <param name="first">  second character (e.g. '�' cedilla)
		/// </param>
		/// <returns>         composite (e.g. '�')
		/// 
		/// </returns>
		public virtual char getPairwiseComposition(char first, char second)
		{
			return (char) compose.Get((first << 16) | second);
		}
		
		/// <summary> Gets recursive decomposition of a character from the 
		/// Unicode Character Database.
		/// </summary>
		/// <param name="canonical">   If true
		/// bit is on in this byte, then selects the recursive 
		/// canonical decomposition, otherwise selects
		/// the recursive compatibility and canonical decomposition.
		/// </param>
		/// <param name="ch">     the source character
		/// </param>
		/// <param name="buffer"> buffer to be filled with the decomposition
		/// 
		/// </param>
		public virtual void  getRecursiveDecomposition(bool canonical, char ch, StringBuilder buffer)
		{
			string decomp = decompose.Get(ch);
			if (decomp != null && !(canonical && isCompatibility.Get(ch)))
			{
				 for (int i = 0; i < decomp.Length; ++i)
				{
					getRecursiveDecomposition(canonical, decomp[i], buffer);
				}
			}
			else
			{
				// if no decomp, append
				buffer.Append(ch);
			}
		}
		
		// =================================================
		//                   PRIVATES
		// =================================================
		
		/// <summary> Only accessed by NormalizerBuilder.
		/// </summary>
		internal NormalizerData(IntHashtable canonicalClass, IntStringHashtable decompose, IntHashtable compose, BitArray isCompatibility, BitArray isExcluded)
		{
			InitBlock();
			this.canonicalClass = canonicalClass;
			this.decompose = decompose;
			this.compose = compose;
			this.isCompatibility = isCompatibility;
			this.isExcluded = isExcluded;
		}
		
		/// <summary> Just accessible for testing.
		/// </summary>
		internal bool getExcluded(char ch)
		{
			return isExcluded.Get(ch);
		}
		
		/// <summary> Just accessible for testing.
		/// </summary>
		internal string getRawDecompositionMapping(char ch)
		{
			return decompose.Get(ch);
		}
		
		/// <summary> For now, just use IntHashtable
		/// Two-stage tables would be used in an optimized implementation.
		/// </summary>
		private IntHashtable canonicalClass;
		
		/// <summary> The main data table maps chars to a 32-bit int.
		/// It holds either a pair: top = first, bottom = second
		/// or singleton: top = 0, bottom = single.
		/// If there is no decomposition, the value is 0.
		/// Two-stage tables would be used in an optimized implementation.
		/// An optimization could also map chars to a small index, then use that
		/// index in a small array of ints.
		/// </summary>
		private IntStringHashtable decompose;
		
		/// <summary> Maps from pairs of characters to single.
		/// If there is no decomposition, the value is NOT_COMPOSITE.
		/// </summary>
		private IntHashtable compose;
		
		/// <summary> Tells whether decomposition is canonical or not.
		/// </summary>
		//UPGRADE_NOTE: The initialization of  'isCompatibility' was moved to method 'InitBlock'. 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="jlca1005"'
		private BitArray isCompatibility;
		
		/// <summary> Tells whether character is script-excluded or not.
		/// Used only while building, and for testing.
		/// </summary>
		
		//UPGRADE_NOTE: The initialization of  'isExcluded' was moved to method 'InitBlock'. 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="jlca1005"'
		private BitArray isExcluded;
	}
}